@@ -17,8 +17,11 @@
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 5, 0)
+#include <linux/security.h>
+#ifndef GRSECURITY_VERSION
 #define get_random_long() (((u64)get_random_int() << 32) | get_random_int())
 #endif
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
 #define RCU_LOCKDEP_WARN(cond, message) rcu_lockdep_assert(!(cond), message)
@@ -116,11 +119,12 @@ __attribute__((unused)) static inline int udp_sock_create_new(struct net *net, s
 #define IP6_ECN_set_ce(a, b) IP6_ECN_set_ce(b)
 #endif
 
-/* https://lkml.org/lkml/2016/9/28/904 */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
 #define time_is_before_jiffies64(a) time_after64(get_jiffies_64(), a)
 #define time_is_after_jiffies64(a) time_before64(get_jiffies_64(), a)
 #define time_is_before_eq_jiffies64(a) time_after_eq64(get_jiffies_64(), a)
 #define time_is_after_eq_jiffies64(a) time_before_eq64(get_jiffies_64(), a)
+#endif
 
 /* https://lkml.org/lkml/2015/6/12/415 */
 #include <linux/netdevice.h>
@@ -129,4 +133,24 @@ static inline struct net_device *netdev_pub(void *dev)
 	return (struct net_device *)((char *)dev - ALIGN(sizeof(struct net_device), NETDEV_ALIGN));
 }
 
+/* https://lkml.org/lkml/2016/10/1/187 */
+#ifdef CONFIG_WIREGUARD_PARALLEL
+#include <linux/padata.h>
+static inline int padata_queue_len(struct padata_instance *pinst)
+{
+	int len;
+	rcu_read_lock_bh();
+	len = atomic_read(&rcu_dereference_bh(pinst->pd)->refcnt);
+	rcu_read_unlock_bh();
+	return len;
+}
+#endif
+
+/* PaX compatibility */
+#ifdef CONSTIFY_PLUGIN
+#include <linux/cache.h>
+#undef __read_mostly
+#define __read_mostly
+#endif
+
 #endif
