@@ -234,7 +234,7 @@ static void setup(struct net_device *dev)
 	dev->addr_len = 0;
 	dev->needed_headroom = DATA_PACKET_HEAD_ROOM;
 	dev->needed_tailroom = noise_encrypted_len(MESSAGE_PADDING_MULTIPLE);
-	dev->type = ARPHRD_NONE;
+	dev->type = ARPHRD_VOID; /* Virtually the same as ARPHRD_NONE, except doesn't get IP6 auto config. */
 	dev->flags = IFF_POINTOPOINT | IFF_NOARP | IFF_MULTICAST;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 3, 0)
 	dev->flags |= IFF_NO_QUEUE;
